#ifdef GL_ES
precision highp float;
#endif

attribute highp vec4 a_position;
attribute highp vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute mediump vec2 a_texCoord1;
attribute mediump vec2 a_texCoord2;
attribute highp vec3 a_normal;
attribute mediump vec3 a_tangent;
attribute mediump vec3 a_binormal;

// x = fog start
// y = fog end
uniform mediump vec4 u_fog_settings; 

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 
// Ambient Color in RGB, intensity in A (contrast)
uniform lowp vec4 u_ambientLightColor;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;
uniform highp mat4 u_viewMatrixInverse;
uniform highp mat4 u_modelViewMatrix;

// DATA OUT TO PIXEL SHADER
varying mediump vec3 v_ambient_colour;
varying highp vec3 v_normal;
varying highp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec2 v_texCoord2;
varying lowp vec3 v_camDir;
varying mediump float v_fog_intensity;

void main() {
	gl_Position = (u_modelViewProjectionMatrix * a_position);
	
    v_normal = ((u_modelMatrix * vec4(a_normal, 0.0))).xyz;
	v_vert_color = a_color;
	v_texCoord0 = a_texCoord;
	v_texCoord1 = a_texCoord1;
	v_texCoord2 = a_texCoord2;
	mediump vec3 ambColour = u_ambientLightColor.rgb * u_ambientLightColor.a;
	// Calculate the hemispherical lighting change as we're not doing any normal maps
	v_ambient_colour = mix(ambColour * 0.25, ambColour, (v_normal.y+1.0)*0.5);
	
	v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);

	mediump float dist = max(0.0, length(abs((u_modelViewMatrix * a_position).xyz)) - u_fog_settings.x);
	v_fog_intensity = clamp(dist/u_fog_settings.y, 0.0, 1.0);
}

