#ifdef GL_ES
precision highp float;
#endif

#include "includes/common_functions.inc"

attribute highp vec4 a_position;
attribute mediump vec2 a_texCoord;
attribute mediump vec2 a_texCoord1;
attribute mediump vec3 a_normal;
attribute mediump vec3 a_tangent;
attribute mediump vec3 a_binormal;
attribute mediump vec4 a_color;

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix;
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor;
// Ambient Color in RGB, intensity in A (contrast)
uniform lowp vec4 u_ambientLightColor;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;

// DATA OUT TO PIXEL SHADER
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;
varying mediump vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec3 v_camDir;

void main() {
	gl_Position = (u_modelViewProjectionMatrix * a_position);
	
    v_normal = ((u_modelMatrix * vec4(a_normal, 0.0))).xyz;
    v_tangent = ((u_modelMatrix * vec4(a_tangent, 0.0))).xyz;
    v_binormal = ((u_modelMatrix * vec4(a_binormal, 0.0))).xyz;
	
	v_texCoord0 = a_texCoord;
	v_texCoord1 = a_texCoord1;

	v_vert_color.rgb = vert_CalculateMRW(a_color.rgb);
	v_vert_color.a = a_color.a;
	
	v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);	
}

// Possible optimistaion notes:
// move envmap coords into Vert shader will lose fidelity due to ignoring normal maps, but shiney surfaces are often smooth
