varying lowp vec4 v_color;
varying lowp vec2 v_texCoord1;

uniform sampler2D u_texture;
uniform sampler2D u_texture1;
uniform vec4 u_options;
uniform vec4 u_burn;
uniform vec4 u_color4;

/*

	u_options
	z: edge smoothness
	w: dissolve uv scale

	u_burn
	x: red color
	y: green color
	z: blue color
	w: burn edge thickness
	
*/

void main() {
	// Options
	float edge_smooth = u_options.z;
	float dissolve_uv_scale = u_options.w;
	vec3 burn_color = u_burn.rgb;
	float burn_edge_thickness = u_burn.w;
	
	// Shader
	float val = (burn_edge_thickness + edge_smooth + 1.0) * v_color.a;
	
	vec4 c = texture2D(u_texture, v_texCoord1);
	float burnTex = texture2D(u_texture1, v_texCoord1 * dissolve_uv_scale).g;
	
	float burn2 = smoothstep(0.0, edge_smooth, -burnTex+val);
	float glow  = smoothstep(burn_edge_thickness, edge_smooth + burn_edge_thickness, -burnTex+val);
	burn2 = clamp(burn2,0.0,1.0);
	glow = clamp(burn2-glow,0.0,1.0);
	c.rgb = mix(c.rgb, u_burn.rgb/255.0, glow);
	
	gl_FragColor = vec4(vec3(c),burn2*c.a);
}