#include "utils/GeometryUtils.inc"

varying lowp vec2 v_mainTexCoord;
varying lowp vec2 v_distortTexRedCoord;
varying lowp float v_distortRedIntensity;
varying lowp vec2 v_distortTexGreenCoord;
varying lowp float v_distortGreenIntensity;
varying lowp vec2 v_dissolveTexCoord;
varying highp float v_dissolve;
varying lowp vec4 v_color;

uniform lowp vec4 u_smoothness_intensity; // smoothness xy, intensity main, intensity dissolve

uniform sampler2D u_texture; // main texture
uniform sampler2D u_texture1; // distort texture
uniform sampler2D u_texture2; // dissolve texture

void main()
{
  vec2 distort;

  //distort in x by red channel
  float distortRed = texture2D(u_texture1, v_distortTexRedCoord).r;
  distort.x = (distortRed - 0.5) * v_distortRedIntensity;

  //distort in y by green channel
  float distortGreen = texture2D(u_texture1, v_distortTexGreenCoord).g;
  distort.y = (distortGreen - 0.5) * v_distortGreenIntensity;

  //distort intensity by static blue channel
  float distortIntensityBlue = texture2D(u_texture1, v_mainTexCoord).b;

  //main color
  vec2 mainTexDistortCoord = v_mainTexCoord + (distort * distortIntensityBlue * u_smoothness_intensity.z);
  vec4 mainTexColor = texture2D(u_texture, mainTexDistortCoord);

  //apply dissolve alpha
  vec2 dissolveTexDistortCoord = v_dissolveTexCoord + distort * distortIntensityBlue * u_smoothness_intensity.w;
  vec4 dissolveColor = texture2D(u_texture2, dissolveTexDistortCoord);
	float dissolveTexValue = dissolveColor.a;
	float dissolveValue = smoothstep(v_dissolve - u_smoothness_intensity.x, v_dissolve + u_smoothness_intensity.y, dissolveTexValue);

	gl_FragColor = vec4(mainTexColor.rgb * v_color.rgb, mainTexColor.a * v_color.a * dissolveValue);
}
