#include "utils/GeometryUtils.inc"

varying lowp vec2 v_mainTexCoord;
varying lowp vec2 v_distortTexRedCoord;
varying lowp float v_distortRedIntensity;
varying lowp vec2 v_distortTexGreenCoord;
varying lowp float v_distortGreenIntensity;
varying lowp vec2 v_maskTexCoord;
varying lowp vec4 v_color;

uniform sampler2D u_texture; // main texture
uniform sampler2D u_texture1; // distort texture
uniform sampler2D u_texture2; // mask texture

uniform lowp vec4 u_intensity; // intensity main, intensity dissolve

void main()
{
  vec2 distort;

  //distort in x by red channel
  float distortRed = texture2D(u_texture1, v_distortTexRedCoord).r;
  distort.x = (distortRed - 0.5) * v_distortRedIntensity;

  //distort in y by green channel
  float distortGreen = texture2D(u_texture1, v_distortTexGreenCoord).g;
  distort.y = (distortGreen - 0.5) * v_distortGreenIntensity;

  //distort intensity by static blue channel
  float distortIntensityBlue = texture2D(u_texture1, v_mainTexCoord).b;

  //main color
  vec2 mainTexDistortCoord = v_mainTexCoord + distort * distortIntensityBlue * u_intensity.x;
  vec4 mainTexColor = texture2D(u_texture, mainTexDistortCoord);

  //mask alpha
  vec2 maskTexDistortCoord = v_maskTexCoord + distort * distortIntensityBlue * u_intensity.y;
  float hasToMak = IsInsideRectangle(maskTexDistortCoord, vec2(0.5), vec2(1.0));
  float maskTexAlpha = texture2D(u_texture2, maskTexDistortCoord).a * hasToMak;

  gl_FragColor = mainTexColor * v_color;
  gl_FragColor.a *= maskTexAlpha;
}
