#include "utils/Utils.inc"

attribute highp vec4 a_position;
attribute lowp vec4 a_color;
attribute lowp vec2 a_texCoord;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp float u_time;
uniform lowp vec4 u_color4; // {red-> mask movement x, green-> mask movement y}

uniform lowp vec4 u_main_scroll_tiling; // {scroll speed x, scroll speed y, tiling x, tiling y}
uniform lowp vec4 u_mask_pos_size; // {pos x, pos y, size x, size y}

varying lowp vec2 v_mainTexCoord;
varying lowp vec2 v_maskTexCoord;
varying lowp vec4 v_color;

void main()
{
	gl_Position = u_modelViewProjectionMatrix * a_position;

	// calculate main UVs
	v_mainTexCoord = CalculateCoordsByTime(
		a_texCoord,
		vec2(0.0),
		u_main_scroll_tiling.zw,
		u_main_scroll_tiling.xy,
		u_time);

	// apply color rg as displacement on the mask
	vec2 maskPos = u_mask_pos_size.xy + u_color4.rg;

	// apply mask position and size
	vec4 maskBounds = vec4(-u_mask_pos_size.zw, vec2(1.0));
	vec2 maskPosNormalized = NormalizeInBounds(maskPos, maskBounds);
	v_maskTexCoord = a_texCoord - maskPosNormalized;
	v_maskTexCoord /= u_mask_pos_size.zw;

	v_color = a_color;
}
