uniform lowp vec4 u_ambientBottomColor;

vec3 rgb2hsv(vec3 c)
{
    mediump vec4 K = vec4(0.0, -0.333, 0.666, -1.0);
    mediump vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    mediump vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    mediump float d = q.x - min(q.w, q.y);
    mediump float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    mediump vec4 K = vec4(1.0, 0.666, 0.333, 3.0);
    // P Needs to be highP to fix black colour errors on iphone
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 vert_CalculateMRW(vec3 vert_color)
{
	mediump vec3 metallic_roughness = vert_color;
	metallic_roughness.g = max(0.001, metallic_roughness.g);
	metallic_roughness.b *= 12.0;  // Artifical boost value for the fake wetness effect
	return metallic_roughness;
}

vec3 vert_AmbientColor(vec4 ambientLightColor, float normalY)
{
	return mix(u_ambientBottomColor.rgb, ambientLightColor.rgb, (normalY + 1.0) * 0.5).rgb;
}

vec3 hsvShiftDetail(vec3 albedo, float detailMask, vec3 u_HSV_shift)
{
	//mediump vec3 HSVColour = rgb2hsv(albedo.rgb);
	//HSVColour.r -= u_HSV_shift.x;
	//HSVColour.g += u_HSV_shift.y;
	//HSVColour.b += u_HSV_shift.z;
	//HSVColour.g = clamp(HSVColour.g, 0.0, 1.0);
	//HSVColour.b = clamp(HSVColour.b, 0.0, 1.0);
	//mediump vec3 shifted = hsv2rgb(HSVColour);
	return mix(albedo.rgb+albedo.rgb*u_HSV_shift.b, albedo.rgb, detailMask);
}
/*
vec3 hsvShiftDetail(vec3 albedo, float detail, vec3 u_HSV_shift, sampler2D hs_tex) 
{
	// calc current albedo hue
	float huemin = min(min(albedo.r,albedo.g), albedo.b);
	float huemax = max(max(albedo.r, albedo.g), albedo.b);
*/	
	// What's going on here?
	// Based on:
	// If Red is max, then Hue = (G-B)/(max-min)
	// If Green is max, then Hue = 2.0 + (B-R)/(max-min)
	// If Blue is max, then Hue = 4.0 + (R-G)/(max-min)
	// (https://stackoverflow.com/questions/23090019/fastest-formula-to-get-hue-from-rgb)
	
	// Using 3 mixes like if statements with 4 different outcomes
	// giving us the proper calculation
	
	// 			  Is R or B max?
	//         /              \
	// Is R or G max?    Is B or G max?
	//    |    |            |    |
	//    R    G            B    G

	// if green-red < 0 ceiled is 0 giving us red
/*	float albedohue = mix(
			mix((albedo.g-albedo.b)/(huemax-huemin), // is red
			2.0+(albedo.b-albedo.r)/(huemax-huemin), // is green
			ceil(albedo.g-albedo.r)), 
			mix(2.0+(albedo.b-albedo.r)/(huemax-huemin), // is green
			4.0+(albedo.r-albedo.g)/(huemax-huemin), // is blue
			ceil(albedo.b-albedo.g)),
		ceil(albedo.b-albedo.r))*0.16666666; // divided by 6 (because math?)
*/	
	// testing super cheap switching divisions with magic numbers
	// seems to work ok if saturation is 1.0
/*	float albedohue = mix(
			mix((albedo.g-albedo.b)*1.1, // is red
			2.0+(albedo.b-albedo.r)*1.1, // is green
			ceil(albedo.g-albedo.r)), 
			mix(2.0+(albedo.b-albedo.r)*1.1, // is green
			4.0+(albedo.r-albedo.g)*0.9, // is blue
			ceil(albedo.b-albedo.g)),
		ceil(albedo.b-albedo.r))*0.16666666; // divided by 6 (because math?)
		
	lowp vec2 hsv_shift_HS = vec2(fract(-u_HSV_shift.x+albedohue),1.0-u_HSV_shift.y);
	float hsv_shift_V = u_HSV_shift.z;
	
	lowp vec3 shifted = texture2D(hs_tex, hsv_shift_HS).rgb + hsv_shift_V;
	albedo.rgb = mix(shifted, albedo, detail);
	
	return vec3(albedo);
}
*/
vec2 vert_CalculateSphericalCoords(float roughness, vec3 R)
{
	// SPHERICAL REFLECTION MAP
	// Using Floor(a+0.5) to replace Round, it isn't avalible in this version of openGL
	float envMapLevel = 0.125 + floor((3.0 * roughness) + 0.5)/4.0;
	float m = 2.0 * sqrt( R.x * R.x + R.y * R.y + (R.z + 1.0) * (R.z + 1.0) );//pow(( R.x * R.x + R.y * R.y + (R.z + 1.0) * (R.z + 1.0) ), 0.5);//sqrt( R.x * R.x + R.y * R.y + (R.z + 1.0) * (R.z + 1.0) );
	lowp vec2 sphericalCoords = R.xy / m;
	sphericalCoords.x = (sphericalCoords.x * 0.246) + envMapLevel;
	sphericalCoords.y = -sphericalCoords.y + 0.5;
	return sphericalCoords;
}

vec4 CalculateEnvSpec(vec4 envMap, vec3 metallic_roughness, float NdotV, vec3 amb_color, vec3 spec_color)
{
	envMap.rgb *= (1.0 - metallic_roughness.g) * (1.0 - NdotV*NdotV) * spec_color * metallic_roughness.b;
	envMap.rgb = clamp(envMap.rgb * 6.28 * amb_color, 0.0001, 1.0); // PR DUPLICATE MUL * metallic_roughness.b; // if this is clamped to 0.0, some faces go full black on some devices
	envMap.a *= (1.0 - NdotV) + 1.0;  //To boost the brightness on the angles as the Mips kick in.
	return vec4(envMap.rgba);
}
vec3 CalculateBRDF(float NdotV, float roughness, vec3 spec_color, vec3 light, float spec_mask, float occlusion)
{
	mediump float spec_BRDF = pow(max(1.0 - NdotV, 0.0), 6.28 - roughness) * max(0.94 - roughness, 0.0);// * light;
	mediump vec3 return_value = (spec_BRDF * light) + (spec_mask * spec_color * 60.0); // 60 is an arbitary scalar value for now
	return_value *=  occlusion;
	return vec3(return_value);
}
/*
float CalculateWorldBend(float bend_amount, vec4 position) //, mat4 modelMatrix, mat4 modelMatrixInverse)
{
	//highp vec4 vWsPos = (modelMatrix * position);
	highp float curvature = bend_amount / 10000.0;

  highp float start_distance = -3500.0;
	mediump float stepResult = step(start_distance, position.z);
	highp float yPos = (((position.z - start_distance)*(position.z - start_distance)) * - curvature) * (1.0-stepResult);

	// add curvature to the vertex
	//vWsPos = vec4( 0.0, yPos , 0.0, 0.0 );

  return yPos; // + (modelMatrixInverse * vWsPos));
}*/

