/*
	Fog
	inputs:
	pos			from shader pos
				what actually describes the fog, for instance -a_position.y or glpos.w
	offset		from u_depFogXoffYrngZmul.x or u_heiFogXoffYrngZmul.x
				start of fog away from fog direction (into the screen). For instance 2000.0
	range		from u_depFogXoffYrngZmul.y or u_heiFogXoffYrngZmul.y
				fog fade range from offset toward fog direction (camera). For instance 1000.0
	strength	from u_depFogXoffYrngZmul.z or u_heiFogXoffYrngZmul.z
				multiplies the fog strength allover, limiting the maximum. For instance 0.7
				
				Depth fog explanation
				camera                      range                   offset
				|--/  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>|  100% fog
				|--\                          |<<<<<<<<<<<<<<<<<<<<<<<|  100% fog
				                                     fading fog
*/

uniform lowp vec4 u_depFogXoffYrngZmul;
uniform lowp vec4 u_heiFogXoffYrngZmul;
uniform lowp vec4 u_fogColor;

float fog(float pos, float offset, float range, float strength) {
	float fog = clamp(min(pos - offset + range, 0.0) + ((pos - offset + range)/range), 0.0, 1.0) * strength;
	return(fog * fog); // exponential fade
}

float depthFog(float pos) {
	return( fog(pos, u_depFogXoffYrngZmul.x, u_depFogXoffYrngZmul.y, u_depFogXoffYrngZmul.z) );
}

float heightFog(float pos) {
	return( fog(pos, u_heiFogXoffYrngZmul.x, u_heiFogXoffYrngZmul.y, u_heiFogXoffYrngZmul.z) );
}