#include "includes/fog.inc"

// User input //
uniform mediump sampler2D u_texture;
uniform mediump sampler2D u_texture1;

// Multiplier for additive detail textures
// x = Detail Shininess
// y = Highlight shininess
// z = Detail repetitions
uniform mediump vec4 u_shine_multiplier;

// Data IN FROM VERTEX SHADER
varying highp vec4 v_color;
varying lowp vec2 v_texCoord0;
varying mediump vec2 v_texCoord1;
varying mediump vec2 v_texCoord2;

varying float v_fog;

void main() {
	
	// ALBEDO - DIFFUSE TEXTURE READ
	mediump vec4 albedo = texture2D(u_texture, v_texCoord0);
	albedo.rgb = albedo.rgb * albedo.rgb; //pow(albedo.rgb, vec3(2.2, 2.2, 2.2));
	
	// DETAIL MAP - FIRST TEXTURE READ
	mediump vec4 detail = texture2D(u_texture1, (v_texCoord1 * u_shine_multiplier.z));
	detail.rgb = detail.rgb * detail.rgb; //pow(detail.rgb, vec3(2.2, 2.2, 2.2));
	
	// DETAIL MAP - SECOND TEXTURE READ
	mediump vec4 highlight = texture2D(u_texture1, (v_texCoord2 * u_shine_multiplier.z));
	highlight.rgb = highlight.rgb * highlight.rgb;//pow(highlight.rgb, vec3(2.2, 2.2, 2.2));

	// BUILD FRAGMENT OUTPUT 
	lowp vec4 returnValue = vec4(albedo.rgb, 1.0);
	returnValue.rgb += albedo.rgb * detail.g * u_shine_multiplier.x; // detail.r before
	returnValue.rgb += albedo.rgb * detail.g * highlight.g * u_shine_multiplier.y;
	
	returnValue.rgb = mix(returnValue.rgb, u_fogColor.rgb, v_fog);
	
	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = sqrt(returnValue.rgb);// pow(returnValue.rgb, vec3(1.0/2.2, 1.0/2.2, 1.0/2.2));
	gl_FragColor = vec4(returnValue.rgb, v_color.a);
}