#ifdef GL_ES
precision highp float;
#endif

#include "includes/world_bend.inc"
#include "includes/fog.inc"

attribute highp vec4 a_position;
attribute highp vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute highp vec2 a_texCoord1;
attribute highp vec2 a_texCoord2;
attribute mediump vec3 a_normal;

//User input for controlling wave behaviour
// x = Scroll speed
// y = wave amplitude
// z = wave frequency
uniform mediump vec4 u_wave_settings;
uniform highp vec4 u_world_bend;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelMatrixInverseTranspose;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform highp mat4 u_viewMatrixInverse;

// Ambient Color in RGB, intensity in A (contrast)
uniform mediump vec4 u_ambientLightColor;

uniform highp float u_time;
	
// DATA OUT TO PIXEL SHADER
varying highp vec4 v_color;
varying lowp vec2 v_texCoord0;
varying mediump vec2 v_texCoord1;
varying mediump vec2 v_texCoord2;

varying float v_fog;

void main() {	
	v_color = a_color;
	
	// Wave movement vertex displacement
	float waveTime = (0.5 * sin((u_time * 5.0 * u_wave_settings.x) + (u_wave_settings.z * a_color.g))) + 0.5;
	vec3 Nn = normalize(vec4(a_normal, 1.0) * u_modelMatrixInverseTranspose).xyz;
	vec4 Po = a_position;
	float ds = waveTime * u_wave_settings.y * a_color.r;
	Po.xyz = Po.xyz + (Nn * ds);

	// WORLD BEND CALCULATIONS
	vec4 glpos = u_modelViewProjectionMatrix * Po;
	gl_Position = world_bend(u_modelViewMatrix * Po, glpos);
	
	//UV scroll
	v_texCoord0 = a_texCoord;	
	v_texCoord1 = a_texCoord1;
	v_texCoord1.y -= mod(u_time * u_wave_settings.x, 1.0);
	
	v_texCoord2 = a_texCoord2;
	v_texCoord2.x += (0.5 * sin(0.5 * u_time * u_wave_settings.x)) + 0.5;
	
	v_fog = clamp(depthFog(glpos.w) + heightFog(-a_position.y), 0.0, 1.0);
}

