// * User input * //
uniform mediump sampler2D u_texture;
uniform mediump sampler2D u_texture1;
uniform mediump sampler2D u_texture2;

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 
// Ambient Color in RGB, intensity in A (contrast)
uniform lowp vec4 u_ambientLightColor;

uniform lowp vec4 u_color4;
uniform lowp vec4 u_materialAmbient;  // Distance effect color
uniform lowp vec4 u_materialSpecular;  // Specular color

uniform mediump vec4 u_settings;

varying mediump vec2 v_texCoord0;
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;
varying mediump vec3 v_camDir;

void main() {
	mediump vec4 baseColor = texture2D(u_texture, v_texCoord0);
	mediump vec3 normalmap = texture2D(u_texture1, v_texCoord0).xyz * 2.0 - 1.0;
	mediump vec4 specGloss = texture2D(u_texture2, v_texCoord0);
	
	specGloss = pow(specGloss, vec4(2.2, 2.2, 2.2, 2.2)); // Linear color
	baseColor = pow(baseColor, vec4(2.2, 2.2, 2.2, 2.2)); // Linear color
		
	mediump vec3 N = normalize(v_normal);
	mediump vec3 T = normalize(v_tangent);
	mediump vec3 B = normalize(v_binormal);
	mediump vec3 V = normalize(v_camDir);
	mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	
		// * NORMAL * //
	mediump mat3 TBN = mat3(T, B, N);
	mediump vec3 Nm = normalize(TBN * normalmap);

	// * DIFFUSE * //
	mediump float diffuseIntensity = clamp(dot(Nm, L), 0.0, 1.0) * u_directionalLightColor.a;
	
	lowp vec3 diffuseLightColor = u_directionalLightColor.rgb;
	lowp vec3 ambientLightColor = u_ambientLightColor.rgb;
	
	//Mix between ambient and diffuse lighting, to avoid very strong ambient
	lowp vec3 v_light_color = mix(ambientLightColor,diffuseLightColor,clamp(diffuseIntensity*u_ambientLightColor.a,0.0,99.0));
	
	mediump vec3 color = v_light_color * baseColor.rgb * u_color4.rgb;
	mediump float alpha = baseColor.a * u_color4.a;
	
	// * SPECULAR * //
	mediump float specularGlossiness = 255.0 * specGloss.g;
	mediump vec3 H = normalize(V + L);
	mediump float NdotH = max(dot(Nm, H), 0.0);
	vec3 specularColor = mix(vec3(1,1,1), u_materialSpecular.rgb, specGloss.b); // this lets us control what gets colored spec
	mediump vec3 specular = specularColor * specGloss.r * pow(NdotH, specularGlossiness) * u_materialSpecular.a;
	
	//Add specular
	color += specular;
	
	// Distance fog effect
	mediump float dist = max(0.0, gl_FragCoord.w * u_settings.y + u_settings.x);
	
	// Add the distance fog effect
	color = color + (dist * u_materialAmbient.rgb * u_materialAmbient.a);

	mediump vec4 final = pow(vec4(color, alpha), vec4(1.0/2.2, 1.0/2.2, 1.0/2.2, 1.0/2.2)); //Back to sRGB
	
	gl_FragColor = final;
}