#ifdef GL_ES
precision highp float;
#endif

#include "includes/common_functions.inc"
#include "includes/world_bend.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

attribute highp vec4 a_position;
attribute mediump vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute mediump vec2 a_texCoord1;
attribute mediump vec2 a_texCoord2;
attribute mediump vec3 a_normal;

// Ambient Color in RGB, intensity in A (contrast)
// Interesting fact, if u_ambientLightColor precision is set to lowp shader won't work on iphone5
uniform mediump vec4 u_ambientLightColor;
uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_projectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;
uniform highp float u_time;	
uniform lowp vec4 u_strength;

// DATA OUT TO PIXEL SHADER
varying lowp vec3 v_ambient_colour;
varying mediump vec3 v_normal;
varying lowp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec2 v_texCoord2;
varying lowp vec3 v_camDir;
varying lowp vec2 v_sphericalCoords;
varying lowp float v_closeDarken;

void main() {
	v_normal = ((u_modelMatrix * vec4(a_normal, 0.0))).xyz;

	v_vert_color.rgb = vert_CalculateMRW(a_color.rgb);
	v_vert_color.a = a_color.a;
	
	v_texCoord0 = a_texCoord;
	v_texCoord1 = a_texCoord1;
	v_texCoord2 = a_texCoord2;
	
	v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);
	
	lowp vec3 R = reflect(-normalize(v_camDir), normalize(v_normal));
	v_sphericalCoords = vert_CalculateSphericalCoords(v_vert_color.g, R);
	
	v_ambient_colour = vert_AmbientColor(u_ambientLightColor, v_normal.y);
	
	//add movement
	float offset = abs(fract(v_vert_color.r + u_time * 0.25 +0.5) *2.0 - 1.0);
	float rounded = (offset * offset * (3.0 - 2.0 * offset)) -0.5;	
	
	float y_offset = abs(fract(v_vert_color.r + u_time * 0.5 +0.5) *2.0 - 1.0);
	float y_rounded = (y_offset * y_offset * (3.0 - 2.0 * y_offset)) -0.5;
	
	vec4 pos = u_modelMatrix * a_position;

	pos.x += rounded * u_strength.x * (1.0-v_vert_color.a) ;
	pos.y += y_rounded * u_strength.y * (1.0-v_vert_color.a);
	pos.z += rounded * u_strength.z * (1.0-v_vert_color.a);
	
	vec4 glpos = (u_projectionMatrix * u_viewMatrix) * pos ;	
	
	// Set Vert colour.a to be the fog
	v_vert_color.a = clamp(depthFog(glpos.w) + heightFog(-a_position.y), 0.0, 1.0);
	
	// older values stored here for backup purposes in case we can't afford
	// or don't want to use the uniforms any more
	//v_vert_color.a = fog(glpos.w, -5.0, 0.0008, 0.2);
	//v_vert_color.a += fog(-a_position.y, 0.0, 0.0015, 1.0);
	v_closeDarken = closeDarken(glpos);
	//v_closeDarken *= v_closeDarken;
	// Bending
	gl_Position = world_bend(u_modelViewMatrix * a_position, glpos);
}