#include "includes/common_functions.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

// * User input * //
uniform sampler2D u_texture;

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 

// Data IN FROM VERTEX SHADER
varying lowp vec4 v_color;
varying lowp vec2 v_texCoord;

varying lowp vec3 v_camDir;
varying lowp vec4 v_vert_color;
varying lowp float v_closeDarken;

void main() {
	// NORMALIZE INPUT VALUES TO ENSURE COUNTER INTERPOLATION ISSUES
	mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	mediump vec3 V = normalize(v_camDir);
	
	// BUILD FRAGMENT OUTPUT  
	lowp vec4 returnValue = v_color * texture2D(u_texture, v_texCoord);
	returnValue.rgb = returnValue.rgb * returnValue.rgb;
	
	// CHEAP FOG
	 float LdotV = (dot(L, V)+1.0) * 1.0;
	 lowp vec3 fogColour = u_fogColor.rgb;
	 returnValue.rgb = mix(returnValue.rgb, fogColour, v_vert_color.a);
	 
	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = sqrt(returnValue.rgb);
	// ADD COLOR tO CLOSEDARKEN
	returnValue.rgb = mix((closeDarkenColor(u_closeDarken)* returnValue.rgb), returnValue.rgb, v_closeDarken);
	gl_FragColor = returnValue;
	}