attribute highp vec4 a_position;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;
attribute mediump vec3 a_normal;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;

uniform highp vec4 ScrollSpeed;
uniform highp vec4 Distortion;
uniform highp vec4 UV_Params;
uniform highp float u_time;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying highp float dotproduct;

void main() {
	
	v_color = a_color;
	v_texCoord = a_texCoord * UV_Params.xy;
	
	lowp vec3 v_view = -normalize(u_modelViewMatrix * a_position).rgb;
	lowp vec3 v_normal = normalize(u_modelViewMatrix * vec4(a_normal, 0.0)).rgb;
	dotproduct = max(0.0, dot(v_view, v_normal));
	
	v_texCoord.x += fract(u_time * ScrollSpeed.x);// * (u_time * ScrollSpeed.w + sin(u_time * ScrollSpeed.z) * ScrollSpeed.w);
	v_texCoord.y += fract(u_time * ScrollSpeed.y);// * (u_time * ScrollSpeed.w + sin(u_time * ScrollSpeed.z) * ScrollSpeed.w);

	// vertex normal distortion
	float s = (sin(u_time * Distortion.z + a_position.y * 1.0) * (Distortion.w * 0.5) * v_color.a);
	vec3 newPos = a_position.xyz + a_normal * s;

	gl_Position = u_modelViewProjectionMatrix * vec4(newPos, 1.0);
}