#ifdef GL_ES
precision highp float;
#endif

#include "includes/common_functions.inc"
#include "includes/world_bend.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

attribute highp vec4 a_position;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform highp mat4 u_viewMatrix;
uniform lowp vec4 u_color4;
uniform lowp vec4 u_materialAmbient;
// Ambient Color in RGB, intensity in A (contrast)
uniform mediump vec4 u_ambientLightColor;

// DATA OUT TO PIXEL SHADER
varying lowp vec3 v_camDir;
varying lowp vec4 v_color;
varying lowp vec4 v_vert_color;
varying mediump vec2 v_texCoord;
varying lowp float v_closeDarken;

void main() {
	v_color = u_color4 * vec4(u_materialAmbient.a, u_materialAmbient.a, u_materialAmbient.a, 1.0);
	v_texCoord = a_texCoord;
    
    v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);
	
	vec4 glpos = u_modelViewProjectionMatrix * a_position;		
	
	// Set Vert colour.a to be the fog
	v_vert_color.a = clamp(depthFog(glpos.w) + heightFog(-a_position.y), 0.0, 1.0);
	v_closeDarken = closeDarken(glpos);
	gl_Position = world_bend(u_modelViewMatrix * a_position, glpos);
}

