varying lowp vec2 v_texCoord;
varying lowp vec2 v_texCoord2;
varying lowp vec4 v_vcolor;

uniform lowp vec4 u_color4;
uniform sampler2D u_texture;
uniform sampler2D u_texture1;
//u_materialAmbient used for rgb: Water Color; a: intensity of Caustics 1;
uniform lowp vec4 u_materialAmbient;
//u_materialDiffuse used for rgb: Foam Color; a: intensity of Caustics 2;
uniform lowp vec4 u_materialDiffuse;

uniform mediump vec4 u_dist_strength;
uniform mediump vec4 u_dist_spray_strength;

varying lowp vec2 scrollUV0;
varying lowp vec2 scrollUVFoam1;
varying lowp vec2 scrollUVFoam2;
varying lowp vec2 scrollUVCaustics1;
varying lowp vec2 scrollUVCaustics2;

varying lowp vec2 v_scrollUV1;
varying lowp vec2 v_scrollUV2;

varying lowp vec2 v_scrollSpray1;
varying lowp vec2 v_scrollSpray2;

/*

The strenghts of the dist_strength can be animated by animating vertex color in timelines

*/

void main()
{
	vec4 dist_strength = u_dist_strength;
	vec2 distortTex1 = texture2D(u_texture1, v_scrollUV1).rg * 2.0-1.0;
	vec2 distortTex2 = texture2D(u_texture1, v_scrollUV2).rg * 2.0-1.0;
	//float distortMask = 1.0; 
	vec2 final_uv_distortion1 = dist_strength.xy*distortTex1.rg;
	vec2 final_uv_distortion2 = dist_strength.zw*distortTex2.rg;
	
	vec4 dist_spray_strength = u_dist_spray_strength;
	vec2 distortTex1_spray = texture2D(u_texture1, v_scrollSpray1).rg * 2.0-1.0;
	vec2 distortTex2_spray = texture2D(u_texture1, v_scrollSpray2).rg * 2.0-1.0;
	vec2 final_uv_distortion_spray1 = dist_spray_strength.xy*distortTex1_spray.rg;
	vec2 final_uv_distortion_spray2 = dist_spray_strength.zw*distortTex2_spray.rg;
	
	float water_op = texture2D(u_texture, scrollUV0+(final_uv_distortion1+final_uv_distortion2)).r;
	float water_spray = texture2D(u_texture, v_texCoord+(final_uv_distortion_spray1+final_uv_distortion_spray2)).g;
	float foam1 = texture2D(u_texture, scrollUVFoam1+0.65*(final_uv_distortion1+final_uv_distortion2)).b;
	float foam2 = texture2D(u_texture, scrollUVFoam2+0.65*(final_uv_distortion1+final_uv_distortion2)).b;
	float caustics1 = texture2D(u_texture1, scrollUVCaustics1+0.05*(final_uv_distortion1+final_uv_distortion2)).b;
	float caustics2 = texture2D(u_texture1, scrollUVCaustics2+0.05*(final_uv_distortion1+final_uv_distortion2)).b;
	
	float foam_mask = clamp(water_spray + foam1 + foam2 + (caustics1 * u_materialAmbient.a) + (caustics2 * u_materialDiffuse.a),0.0,1.0);
	//float foam_mask = clamp(foam1 + foam2 + (caustics1 * u_materialAmbient.a) + (caustics2 * u_materialDiffuse.a),0.0,1.0);
	vec3 water_color = u_materialAmbient.rgb;
	vec3 foam_color = u_materialDiffuse.rgb;
	vec3 water_all = mix(water_color,foam_color,foam_mask);
	
	gl_FragColor = vec4(water_all, water_op);
}