// USER INPUT
uniform lowp sampler2D u_texture;
uniform lowp sampler2D u_texture1;
uniform highp vec4 u_glitterPos;
uniform lowp vec4 u_shadowcolor;

// DATA IN FROM VERTEX SHADER
varying lowp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying highp vec2 v_texCoord1;
varying highp vec2 v_texCoord2;
varying highp vec4 v_viewPos;

void main() {   
    // Nice way of generating a noisy texture only where two textures overlap
    float water_noise = texture2D(u_texture, v_texCoord1).r;
    float water_noise2 = texture2D(u_texture, v_texCoord2).r;
    float water_noise3 = clamp((water_noise - 0.5) + (water_noise2 - 0.5), 0.0, 1.0);

    // Position the glitter and set its' size
    highp vec2 glitterViewPos = v_viewPos.xy + u_glitterPos.xy;
    float glitter = 1.0 - clamp(length(glitterViewPos.xy) * u_glitterPos.z, 0.0, 1.0);

    // Adjust intensity
    float glitterPatches = glitter * u_glitterPos.w;
    glitterPatches *= water_noise3;

    // Lightmap
    lowp vec4 lmap = texture2D(u_texture1, v_texCoord0);
    lmap.rgb += u_shadowcolor.rgb;
    lmap = clamp(lmap, 0.0, 1.0);    
    lmap *= v_vert_color;

    // Build pixel output
    lowp vec4 result = lmap + vec4(vec3(1.0) * glitterPatches, glitterPatches);

    // Result
    gl_FragColor = result;
}