#ifdef GL_ES
precision highp float;
#endif

#include "includes/world_bend.inc"

attribute highp vec4 a_position;
attribute lowp vec4 a_color;
attribute lowp vec2 a_texCoord;

// USER INPUT
uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;

uniform float u_time;
uniform lowp vec4 u_color4;
uniform highp vec4 u_distance;

// DATA OUT TO PIXEL SHADER
varying lowp vec2 v_texCoord0;
varying highp vec2 v_texCoord1;
varying highp vec2 v_texCoord2;
varying lowp vec4 v_vert_color;
varying highp vec4 v_viewPos;

void main() {
    // Lightmap UVs
    v_texCoord0 = a_texCoord;

    // Water bobbing
    highp vec4 position = a_position;
    position.y += (cos(u_time) * 5.0);

    // World bend calculations
    vec4 cs_position = u_modelViewProjectionMatrix * position; // Camera space
    gl_Position = world_bend(u_modelViewMatrix * position, cs_position);

    // Alpha to opaqueness fade
    //float dist = distance(vec3(0.0), cs_position.xyz); // NEW
    //float fade = clamp(dist / u_distance.x, 0.0, 1.0);
    float fade = max(0.0, ((cs_position.w - u_distance.x) * 0.001) * 0.4); // OLD

    // Color based on the fade
    v_vert_color.rgb = u_color4.rgb; // Solid
    //v_vert_color.rgb = mix(u_color4.rgb, u_color4.rgb, fade; // Color fade
    //v_vert_color.rgb = mix(vec3(1.0), vec3(0.0), fade); // Color test
	
    // Alpha
    //v_vert_color.a = 1.0; // Solid
    //v_vert_color.a = fade * u_color4.a; // Fade + Alpha
    v_vert_color.a = fade; // Fade

    // Add shoreline edge color
    v_vert_color += a_color.r;

    // Offset the noise UVs
    v_texCoord1 = a_position.xz * 0.0025;
    v_texCoord1.x += fract(u_time * 0.1);
    v_texCoord2 = a_position.xz * 0.003;
    v_texCoord2.y += fract(-u_time * 0.12);

    v_viewPos = u_modelViewMatrix * a_position;
}